// SPDX-FileCopyrightText: Matteo Settenvini <matteo.settenvini@montecristosoftware.eu>
// SPDX-License-Identifier: EUPL-1.2

use {std::env, std::path::PathBuf};

fn main() {
    println!("cargo:rerun-if-changed=wrapper.hpp");

    // Required by NSS
    println!("cargo:rustc-cdylib-link-arg=-Wl,-soname,libnss_parental_ctrls.so.2");

    let bindings = bindgen::Builder::default()
        .header("wrapper.hpp")
        .parse_callbacks(Box::new(bindgen::CargoCallbacks::new()))
        .newtype_enum("nss_status")
        .allowlist_type("nss_status")
        .newtype_enum("HErrno")
        .allowlist_type("HErrno")
        .newtype_enum("EaiRetcode")
        .allowlist_type("EaiRetcode")
        .allowlist_type("gaih_addrtuple")
        .allowlist_function("__nss_configure_lookup")
        .generate()
        .expect("Unable to generate bindings");

    let out_path = PathBuf::from(env::var("OUT_DIR").unwrap());
    bindings
        .write_to_file(out_path.join("bindings.rs"))
        .expect("Couldn't write bindings!");
}
